@@ -4,23 +4,16 @@
 
 extern int verbose;
 
-int  cmd_dumptree(int argc, char *argv[]);
 int  cmd_login(int argc, char *argv[]);
 int  cmd_logout(int argc, char *argv[]);
 int  cmd_lookup(int argc, char *argv[]);
-#ifdef APPLE
 int  cmd_status(int argc, char *argv[]);
-#endif
 int  cmd_print(int argc, char *argv[]);
 int  cmd_view(int argc, char *argv[]);
 void login_usage(void);
 void logout_usage(void);
 void lookup_usage(void);
-#ifdef APPLE
 void status_usage(void);
-#endif
 void print_usage(void);
 void view_usage(void);
-#ifdef APPLE
 extern int loadsmbvfs();
-#endif
